import { CoreService } from './../../../core/services/core.service';
import { CookieService } from 'ngx-cookie-service';
import { Component, ViewEncapsulation, ViewChild, Injector } from '@angular/core';
import { extend } from '@syncfusion/ej2-base';
// tslint:disable-next-line:max-line-length
import {
  EventSettingsModel,
  View,
  GroupModel,
  TimelineMonthService,
  DayService,
  ResizeService,
  WorkHoursModel,
  DragAndDropService,
  ResourceDetails,
  ScheduleComponent,
  TimeScaleModel} from '@syncfusion/ej2-angular-schedule';
import { L10n } from '@syncfusion/ej2-base';
import { sampleData } from './data';
import { Appointment } from '../../models/appointment.model';
import { NgbModal, NgbModalOptions } from '@ng-bootstrap/ng-bootstrap';

import { Patient } from '../../models/patient.model';
import { DatePipe } from '@angular/common';
import { ReadPayloadConstructorUtil } from '../../../core/utils/read-payload-constructor.util';
import { SortModel } from '../../../core/models/crud/sort.model';
import { CriteriaModel } from '../../../core/models/crud/criteria.model';
import { APPOINTMENT_LIST, USER_GRID, CENTER_SERVICES_LIST } from '../../config/patient.config';
import { CoreConstant } from '../../../core/constants/core.constant';
import { PatientConstant } from '../../constants/patient.constant';
import { PatientAppointment } from '../../models/patient-appointment.model';
import { AppointmentFormComponent } from './appointment-form/appointment-form.component';

L10n.load({
  'en-US': {
    schedule: {
      saveButton: 'Save',
      cancelButton: 'Cancel',
      deleteButton: 'Remove',
      newEvent: 'New Appointment'
    }
  }
});

@Component({
  selector: 'ody-appointment',
  templateUrl: './appointment.component.html',
  styleUrls: ['./appointment.component.scss'],
  encapsulation: ViewEncapsulation.None,
  // tslint:disable-next-line:max-line-length
  providers: [DayService, TimelineMonthService, ResizeService, DragAndDropService]
})
export class AppoinmentComponent extends CoreService {
  @ViewChild('scheduleObj', {})
  public scheduleObj: ScheduleComponent;

  public data: object[] = extend([], sampleData, null, true) as object[];
  public selectedDate: Date = new Date();
  public currentView: View = 'Day';
  public workHours: WorkHoursModel = { start: '00:00', end: '23:59' };
  public timeScale: TimeScaleModel = { enable: true, interval: 60, slotCount: 2 };
  public timezone = 'Asia/Kolkata';

  public appointment: Appointment;
  public isDisabled = true;
  public isPatientFound = true;
  public centerServicesData;
  public appointmentData = [];
  public calendervisible = false;
  public selectedServiceType;

  // tslint:disable-next-line:max-line-length
  public modalityDataSource: object[] = [];

  public group: GroupModel = { enableCompactView: false, resources: ['Modalities'] };
  public allowMultiple = false;
  public eventSettings: EventSettingsModel = {
    dataSource: <Object[]>extend([], this.modalityDataSource, null, true),
    fields: {
      subject: { title: 'Patient Name', name: 'patientName' },
      startTime: { title: 'From', name: 'startTime' },
      endTime: { title: 'To', name: 'endTime' },
      description: { title: 'Modality Type', name: 'modalityType' },
      location: { title: 'Center', name: 'stationName' }
    }
  };

  constructor(public _injector: Injector, private modalService: NgbModal, private cookieService: CookieService, private datePipe: DatePipe) {
    super(_injector);
    this.getCenterServicesData();
  }

  public onPopupOpen(args) {
    if (args.type === 'Editor' || args.type === 'QuickInfo')  {
      args.cancel = true;
    }
    // if (args.data.name === 'cellClick') {
    //   if ((args.data.startTime) < new Date(new Date().setHours(0, 0, 0, 0))) {
    //     args.cancel = true;
    //   }
    // } else {
    //   if ((args.data.startTime) < new Date(new Date().setHours(0, 0, 0, 0))) {
    //     args.cancel = true;
    //   }
    // }
  }

  // public onRenderCell(args) {
  //   if (args.date < new Date(new Date().setHours(0, 0, 0, 0))) {
  //     args.element.classList.add('e-disable');
  //     // args.element.classList.add('e-block-appointment');
  //   }
  // }

  // public onActionBegin(args) {
  //   if (args.requestType === 'eventChange') {
  //     if ((args.data.StartTime) < new Date(new Date().setHours(0, 0, 0, 0))) {
  //       args.cancel = true;
  //     }
  //   }
  //   if (args.requestType === 'eventCreate') {
  //     for (let i = 0; i < args.data.length; i++) {
  //       if ((args.data[i].StartTime) < new Date(new Date().setHours(0, 0, 0, 0))) {
  //         args.cancel = true;
  //       }
  //     }
  //   }
  // }

  public onActionComplete(args) {
    if (args.requestType === 'toolBarItemRendered') {
      // this.scheduleObj.element.querySelectorAll('.e-schedule-toolbar .e-date-range')[0].click();
      // this.scheduleObj.element.querySelectorAll('.e-calendar')[0].ej2_instances[0].min = new Date();
      // this.scheduleObj.element.querySelectorAll('.e-schedule-toolbar .e-date-range')[0].click();
    }
    // if (args.requestType === 'dateNavigate') {
    //   if (this.scheduleObj.selectedDate <= this.selectedDate) {
    //     args.cancel = (args.event.target.classList.contains('e-icon-prev'));
    //   }
    // }
  }

  public getCenterServicesData() {
    const sortArray = new Array<SortModel>();
    sortArray.push(ReadPayloadConstructorUtil.getSortWithDefaults('centerId', '-1'));
    const criteria = new Array<CriteriaModel>();
    const clonedObj = Object.assign({}, CENTER_SERVICES_LIST);

    // tslint:disable-next-line:max-line-length
    criteria.push(ReadPayloadConstructorUtil.getCriteriaWithAdditionalFields(
      { centerId: this.cookieService.get('centerId')
      }, clonedObj, this._keycloakService));
    // tslint:disable-next-line:max-line-length
    ReadPayloadConstructorUtil.getReadModalForForm(this._crudService, this._keycloakService, CoreConstant.MODULE_ODYSSEY,
      PatientConstant.APPOINTMENT_CRUD_ID, clonedObj, criteria, sortArray, null).then(response => {
      if (response !== undefined) {
        this.centerServicesData = response.vCenterServices[0];
        this.modalityDataSource = response.vCenterServices[0];
        if (this.centerServicesData !== undefined) {
          this.selectedServiceType = this.centerServicesData[0].serviceType;
          this.getAppointmentData();
        }
      }
    });
  }
  public getAppointmentData() {
    const sortArray = new Array<SortModel>();
    sortArray.push(ReadPayloadConstructorUtil.getSortWithDefaults('centerId', '-1'));
    const criteria = new Array<CriteriaModel>();
    const clonedObj = Object.assign({}, APPOINTMENT_LIST);

    const currentDate = this.datePipe.transform(this.selectedDate, 'yyyy-MM-dd');
    const startDatetemp =  'GREATER THAN OR EQUAL~' + currentDate + ' 00:00:00';
    const endDatetemp = 'LESS THAN OR EQUAL~' + currentDate + ' 23:59:59';
    const statusFlag = 'IN~' + 'SCHEDULED,RESCHEDULED';

    // tslint:disable-next-line:max-line-length
    criteria.push(ReadPayloadConstructorUtil.getCriteriaWithAdditionalFields(
      { centerId: this.cookieService.get('centerId'), statusFlag: statusFlag,
        startTime: startDatetemp, endTime: endDatetemp,
        serviceType: this.selectedServiceType
      }, clonedObj, this._keycloakService));
    // tslint:disable-next-line:max-line-length
    ReadPayloadConstructorUtil.getReadModalForForm(this._crudService, this._keycloakService, CoreConstant.MODULE_ODYSSEY,
      PatientConstant.APPOINTMENT_CRUD_ID, clonedObj, criteria, sortArray, null).then(response => {
      if (response !== undefined) {
        this.appointmentData = response.tPatientAppointments[0];
        if (this.appointmentData !== undefined && this.appointmentData.length > 0) {
          this.appointmentData.forEach(appointmentObj => {
            appointmentObj['Id'] = appointmentObj['opaId'];
            appointmentObj['startTime'] = new Date(appointmentObj['startTime'].substring(0, 19)); // 2019-11-03T01:00:00
            appointmentObj['endTime'] = new Date(appointmentObj['endTime'].substring(0, 19));
          });
        }
        this.eventSettings.dataSource = this.appointmentData;
        this.calendervisible = true;
        if (this.scheduleObj !== undefined) {
          this.scheduleObj.eventSettings.dataSource = this.appointmentData;
          this.scheduleObj.refreshEvents();
        }
      }
    });
  }

  public getServiceName(value: ResourceDetails): string {
    return (value as ResourceDetails).resourceData[(value as ResourceDetails).resource.textField] as string;
  }

  public getModalityDesignation(value: ResourceDetails): string {
    return (value as ResourceDetails).resourceData.description as string;
  }

  public getModalityImageName(value: ResourceDetails): string {
    return this.getServiceName(value).toLowerCase();
  }
  // public onPopupOpen(args: PopupOpenEventArgs): void {
  //   if (args.type === 'Editor' || args.type === 'QuickInfo')  {
  //       args.cancel = true;
  //   }
  // }

  public onNavigation(args) {
    this.selectedDate = args.currentDate;
    this.getAppointmentData();
  }

  public onServiceTypeSelection(serviceType) {
    this.selectedServiceType = serviceType;
    this.getAppointmentData();
  }

  public onTimeSelection(args: any): void {
    // args.requestType = 'eventSelect' when clicked on the scheduled appointment
    if (args.data !== undefined) {
      // const currentAeTitle = args.data.aeTitle instanceof Array ? args.data.aeTitle[0] : args.data.aeTitle;
      let centerService = [];
      if (this.centerServicesData !== undefined && this.centerServicesData.length > 0) {
        const temp = this.centerServicesData.filter(element => element['serviceType'] === this.selectedServiceType);
        if (temp !== undefined && temp.length > 0) {
          centerService = temp[0];
        }
      }
      this.onAppointmentCrud(centerService, args);
    }
  }

  public async onAppointmentCrud(centerService, event) {
    const pipe = new DatePipe('en-US'); // Use your own locale
    // tslint:disable-next-line:max-line-length
    this.appointment = new Appointment();
    let flag = false;
    if (event.requestType === 'eventSelect') {
      flag = true;
      const sortArray = new Array<SortModel>();
      sortArray.push(ReadPayloadConstructorUtil.getSortWithDefaults('modifiedDate', '-1'));

      const criteria = new Array<CriteriaModel>();
      criteria.push(ReadPayloadConstructorUtil.getCriteriaWithDefaults(this._keycloakService));
      const criteriaDatasourceType = new CriteriaModel();
      criteriaDatasourceType.setField('id');
      criteriaDatasourceType.setFunctionName('EQUALS');
      criteriaDatasourceType.setValue(event.data.mrdId);
      criteria.push(criteriaDatasourceType);
      this._auditLog.setScreenName('Patient -> Appointment -> Patient Search');
      // tslint:disable-next-line: max-line-length
      const response = await ReadPayloadConstructorUtil.getData(this._crudService, this._keycloakService, CoreConstant.MODULE_ODYSSEY, PatientConstant.APPOINTMENT_CRUD_ID,
        PatientConstant.APPOINTMENT_PATIENT_ENTITY_ID, USER_GRID, criteria, sortArray, null, this._auditLog);

      if (response.status === 'SUCCESS') {
        if (response.data !== undefined && response.data.length > 0) {
          this.appointment.tPatient = response.data[0];
          this.appointment.tPatientAppointments = event.data;
        }
      }
    } else {
      this.appointment.tPatient = new Patient();
      this.appointment.tPatientAppointments = new PatientAppointment();
    }
    const options: NgbModalOptions = { windowClass: 'modal-xl animate', backdrop: 'static', keyboard: false };
    const modelRef = this.modalService.open(AppointmentFormComponent, options);
    modelRef.componentInstance.enableRemove = flag;
    modelRef.componentInstance.inputData = { appointment: this.appointment, centerService: centerService,
      eventStartTime: pipe.transform(event.data.startTime, 'yyyy-MM-dd HH:mm:ss'),
      eventEndTime: pipe.transform(event.data.endTime, 'yyyy-MM-dd HH:mm:ss')};
    modelRef.componentInstance.modelRef = modelRef;
    modelRef.componentInstance.calenderInstance = this.scheduleObj;
    modelRef.result.then(data => {
      if (data !== undefined && data === 'refreshAppointments') {
        this.getAppointmentData();
      }
    });
  }
}
